/* ----------------------------------------------------------------------------
*
* Raima Database Manager
*
* Copyright (c) 2013 Raima Inc.,  All rights reserved.
*
* Use of this software, whether in source code format, or in executable,
* binary object code form, is governed by the Raima LICENSE which
* is fully described in the LICENSE.TXT file, included within this
* distribution of files.
*
* ----------------------------------------------------------------------------
*/

/* rdm.h is required in all files using the RDM Database */
#include "rdm.h"

/* CursorDirtyRead_db.h was generated from the DDL file CursorDirtyRead_db.ddl. 
* See subsequent examples for DDL modification.
*/
#include "CursorDirtyRead_db.h"

/* CursorDirtyRead_db_dbd.h was generated from the DDL file CursorDirtyRead_db.ddl by the 
* ddlp utility. It is required to use the d_open_ptr API.
*/
#include "CursorDirtyRead_db_dbd.h"

/* EXAMPLE - Hello World with Dirty Read using Cursors */
/* Each time you run this example, a new record is created in the
* database, then all existing records are read from the database
* and printed.
*
* The DDL (Database Definition Language) is in CursorDirtyRead_db.ddl. The database
* has been pre-built and initialized.
*
* The following RDM Functions are illustrated:
*      d_opentask/d_closetask
*      d_open/d_iclose
*      d_cursorAlloc/dc_cursorFree
*      d_trbegin/d_trend
*      d_createRecord
*      d_fillnew
*      dc_getFields
*      dc_first/dc_next
*      d_recread
* They are fully documented in http://www.raima.com/support/online-documentation/.
*
* Each of these functions returns an integer status code, where the
* value S_OKAY indicates a successful call.
*
* For simplicity, this example does not check all return codes, but good
* programming practices would dictate that they are checked after each
* RDM call.
*
* The actual database is stored in a directory named 'CursorDirtyRead_db' in this
* project directory.
*/

int32_t main (void) 
{
    RDM_TASK task;
    int32_t iStatus;

    /* Create the context for the database operations */
    iStatus = d_opentask (&task);
    if (iStatus == S_OKAY)
    {
        /* Open the database for exclusive access */
        iStatus = d_open_ptr ("CursorDirtyRead_db", "x", &CursorDirtyRead_db_dbd, sizeof (CursorDirtyRead_db_dbd), task);

        if (iStatus == S_OKAY)
        {
            struct HelloRecord Hello_input;
            RDM_CURSOR cursor = NULL;

            iStatus = d_initialize (task, CURR_DB);

            if (iStatus == S_OKAY)
                iStatus = d_cursorAlloc (&cursor, task, 0);

            /* Create a record in the new database */
            strncpy (Hello_input.message, "Hello World!", SIZEOF_MESSAGE);

            if (iStatus == S_OKAY)
            {
                /* Start a Insert database transaction. */
                iStatus = d_trbegin ("Dirty Read", task);
                if (iStatus == S_OKAY)
                {
                    uint32_t counter;

                    iStatus = d_reclock (HELLORECORD, "w", task, 0);

                    for (counter = 0; counter < sizeof (Hello_input) / sizeof (struct HelloRecord); counter++)
                    {
                        /* Create a record in the new database */
                        iStatus = d_createRecord (HELLORECORD, &Hello_input,
                            sizeof(Hello_input), cursor, task,
                            dc_getDBN (cursor));
                        /* Start Retrieving data from the Database. */
                        if (iStatus == S_OKAY)
                        {
                            /* Scan and read all created records.
                            * One additional record will be created each time you run this example.
                            */
                            iStatus = d_getRecords (HELLORECORD, cursor, task,
                                dc_getDBN (cursor));
                        }
                        if (iStatus == S_OKAY)
                            iStatus = dc_first (cursor);

                        while (iStatus == S_OKAY)
                        {
                            struct HelloRecord recS;
                            iStatus = dc_getFields (cursor, HELLORECORD, &recS,
                                sizeof(recS), NULL);
                            if (iStatus == S_OKAY)
                            {
                                printf ("%s\n", recS.message);
                                iStatus = dc_next (cursor);
                            }
                        }
                    }

                    iStatus = d_trend (task);
                } 

                iStatus = dc_cursorFree (cursor);
            }
            /* Close the database */
            if (iStatus == S_OKAY)
                iStatus = d_iclose (task, CURR_DB);
            else
                d_iclose (task, CURR_DB);
        }

        if (iStatus == S_OKAY)
            iStatus = d_closetask (task);
        else
            d_closetask (task);
    }

    printf ("Press <Enter> to continue . . .\n");
    getchar ();
}
